@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_conn.c,v 1.28.2.1 2002/11/14 10:05:23 wensong Exp $
+ * Version:     $Id: ip_vs_conn.c,v 1.28.2.2 2003/04/11 14:02:35 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -709,9 +709,8 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 			skb->csum = csum_partial(h.raw, size, 0);
 
 		case CHECKSUM_HW:
-			if (csum_tcpudp_magic(iph->saddr, iph->daddr,
-					      size, iph->protocol, skb->csum))
-			{
+			if (csum_tcpudp_magic(iph->saddr, iph->daddr, size,
+					      iph->protocol, skb->csum)) {
 				IP_VS_DBG_RL("Incoming failed %s checksum "
 					     "from %d.%d.%d.%d (size=%d)!\n",
 					     ip_vs_proto_name(iph->protocol),
@@ -719,7 +718,10 @@ static int ip_vs_nat_xmit(struct sk_buff *skb, struct ip_vs_conn *cp)
 					     size);
 				goto tx_error;
 			}
-		default:/* CHECKSUM_UNNECESSARY */
+			break;
+		default:
+			/* CHECKSUM_UNNECESSARY */
+			break;
 		}
 	}
 
