@@ -5,7 +5,7 @@
  *              high-performance and highly available server based on a
  *              cluster of servers.
  *
- * Version:     $Id: ip_vs_ctl.c,v 1.30 2002/07/11 14:26:41 wensong Exp $
+ * Version:     $Id: ip_vs_ctl.c,v 1.30.2.1 2002/11/14 10:05:23 wensong Exp $
  *
  * Authors:     Wensong Zhang <wensong@linuxvirtualserver.org>
  *              Peter Kese <peter.kese@ijs.si>
@@ -1922,10 +1922,14 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		sprintf(buf, "IP Virtual Server version %d.%d.%d (size=%d)",
 			NVERSION(IP_VS_VERSION_CODE), IP_VS_CONN_TAB_SIZE);
-		if (*len < strlen(buf)+1)
-			return -EINVAL;
-		if (copy_to_user(user, buf, strlen(buf)+1) != 0)
-			return -EFAULT;
+		if (*len < strlen(buf)+1) {
+			ret = -EINVAL;
+			goto out;
+		}
+		if (copy_to_user(user, buf, strlen(buf)+1) != 0) {
+			ret = -EFAULT;
+			goto out;
+		}
 		*len = strlen(buf)+1;
 	}
 	break;
@@ -1937,7 +1941,7 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 		info.size = IP_VS_CONN_TAB_SIZE;
 		info.num_services = ip_vs_num_services;
 		if (copy_to_user(user, &info, sizeof(info)) != 0)
-			return -EFAULT;
+			ret = -EFAULT;
 	}
 	break;
 
@@ -1947,14 +1951,18 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		if (*len < sizeof(get)) {
 			IP_VS_ERR("length: %u < %u\n", *len, sizeof(get));
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
+		}
+		if (copy_from_user(&get, user, sizeof(get))) {
+			ret = -EFAULT;
+			goto out;
 		}
-		if (copy_from_user(&get, user, sizeof(get)))
-			return -EFAULT;
 		if (*len != (sizeof(get)+sizeof(struct ip_vs_service_user)*get.num_services)) {
 			IP_VS_ERR("length: %u != %u\n", *len,
 				  sizeof(get)+sizeof(struct ip_vs_service_user)*get.num_services);
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
 		}
 		ret = __ip_vs_get_service_entries(&get, user);
 	}
@@ -1967,10 +1975,13 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		if (*len != sizeof(get)) {
 			IP_VS_ERR("length: %u != %u\n", *len, sizeof(get));
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
+		}
+		if (copy_from_user(&get, user, sizeof(get))) {
+			ret = -EFAULT;
+			goto out;
 		}
-		if (copy_from_user(&get, user, sizeof(get)))
-			return -EFAULT;
 
 		if (get.fwmark)
 			svc = __ip_vs_svc_fwm_get(get.fwmark);
@@ -1998,15 +2009,19 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		if (*len < sizeof(get)) {
 			IP_VS_ERR("length: %u < %u\n", *len, sizeof(get));
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
+		}
+		if (copy_from_user(&get, user, sizeof(get))) {
+			ret = -EFAULT;
+			goto out;
 		}
-		if (copy_from_user(&get, user, sizeof(get)))
-			return -EFAULT;
 		if (*len != (sizeof(get) +
 			     sizeof(struct ip_vs_dest_user)*get.num_dests)) {
 			IP_VS_ERR("length: %u != %u\n", *len,
 				  sizeof(get)+sizeof(struct ip_vs_dest_user)*get.num_dests);
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
 		}
 		ret = __ip_vs_get_dest_entries(&get, user);
 	}
@@ -2018,7 +2033,8 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		if (*len < sizeof(u)) {
 			IP_VS_ERR("length: %u < %u\n", *len, sizeof(u));
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
 		}
 		__ip_vs_get_timeouts(&u);
 		if (copy_to_user(user, &u, sizeof(u)) != 0)
@@ -2032,7 +2048,8 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 
 		if (*len < sizeof(u)) {
 			IP_VS_ERR("length: %u < %u\n", *len, sizeof(u));
-			return -EINVAL;
+			ret = -EINVAL;
+			goto out;
 		}
 		u.state = ip_vs_sync_state;
 		strcpy(u.mcast_ifn, ip_vs_mcast_ifn);
@@ -2045,6 +2062,7 @@ do_ip_vs_get_ctl(struct sock *sk, int cmd, void *user, int *len)
 		ret = -EINVAL;
 	}
 
+  out:
 	up(&__ip_vs_mutex);
 	return ret;
 }
